% Questo script analizza alcuni tipi di segnali sinusoidali, con lo
% scopo di calcolarne la frequenza. L'analisi viene effettuata sia
% utilizzando una finestra rettangolare, sia una di tipo Hanning.
%
% Per essere eseguito richiede la presenza del Signal Processing Toolbox
% Lo zoom viene effettuato in modo automatico, e potrebbe essere
% necessario modificarlo.
%
% Autore: Riccardo Riva
% Versione: 1.0
% Data: 18 ottobre 2011


close all; clear all; clc;

% impostazioni
N  = 6000    % numero di campioni
fs = 200     % frequenza di campionamento [Hz]
% e' interessante osservare come variano i risultati modificando la
% frequenza di campionamento anche di 10 volte

% scelgo un segnale sinusoidale con rumore
dt = 1/fs;
T0 = dt*(N-1)       % periodo dell'osservazione [s]
t(:,1) = 0:dt:T0;
% segnale con frequenza costante di valore 10 Hz
% segnale = sin(2*pi*10*t) + randn(size(t))*3;
% segnale con frequenza che oscilla nel tempo attorno a 10 Hz
segnale = sin(2*pi*(10+0.3*sin(0.1.*t)).*t) + randn(size(t))*3;
% segnale con frequenza che aumenta nel tempo, a partire da 10 Hz
% segnale = sin(2*pi*(10+0.05.*t).*t) + randn(size(t))*3;

% disegno il segnale
figure();
hold on;
title('Segnale')
plot(t, segnale);
xlabel('tempo [s]');
ylabel('Ampiezza [V]');

% eseguo la trasformata di Fourier
F = fft(segnale);
% determino le frequenze
n = length(F);
f = (0:n-1)/(dt*n);
% calcolo la potenza del segnale
potenza = F.*conj(F)/n;

% ripeto l'analisi con una finestratura di tipo Hanning
finestra = hanning(N, 'periodic');
segnaleFinestrato = segnale .* finestra;
Ffinstrata = fft(segnaleFinestrato);
potenzaFinstrata = Ffinstrata.*conj(Ffinstrata)/n;

% disegno le trasformate di Fourier
figure();
subplot(2, 1, 1);
hold on;
title('Trasformata di Fourier con finestra rettangolare')
plot(f, potenza);
ylabel('Densita di potenza');
subplot(2, 1, 2);
hold on;
title('Trasformata di Fourier con finestra Hanning')
plot(f, potenzaFinstrata);
xlabel('Frequenza [Hz]');
ylabel('Densita di potenza [dB/Hz]');

% eseguo lo zoom
[dummy indice] = max(potenzaFinstrata);
indice = indice * 2;
figure();
subplot(2, 1, 1);
hold on;
title('Trasformata di Fourier con finestra rettangolare')
plot(f(1:indice), potenza(1:indice));
ylabel('Densita di potenza');
subplot(2, 1, 2);
hold on;
title('Trasformata di Fourier con finestra Hanning')
plot(f(1:indice), potenzaFinstrata(1:indice));
xlabel('Frequenza [Hz]');
ylabel('Densita di potenza [dB/Hz]');



