% Questo script costruisce una popolazione di 400 misure, e ne calcola
% media, varianza e deviazione standard. Successivamente divide le misure
% in blocchi e calcola le stesse quantita', per poi mediare i risultati.
%
% Per essere eseguito richiede la presenza di:
% - Statistics Toolbox
% - Signal Processing Toolbox
%
% Autore: Riccardo Riva
% Versione: 1.0
% Data: 18 ottobre 2011


close all; clear all; clc;

% creo una distribuzione gaussiana di 400 dati compresa (circa) fra 340+-10
dati = normrnd(340, 10/3, 400, 1);
% altri tipi di distribuzioni, con caratteristiche simili
% dati = 330 + (350-330).*rand(400, 1);
% dati = poissrnd(340, 400, 1);
% dati = exprnd(340, 400, 1);

% calcolo la media, deve risultare circa 340 m/s
media = mean(dati)
% calcolo la varianza
% la flag 0 indica che uso la definizione con N-1
varianza = var(dati, 0)
% calcolo la deviazione standard (radice quadrata della varianza)
% di default la flag e' 0
devStandard = std(dati)

% plotto i risultati in un istogramma, con fasce da 1 m/s
% in alternativa: Nbins = 1.87*(length(dati)-1)^0.4+1 = 21.5
% dopodiche' osservo quanto la distribuzione e' normale
figure();
subplot(2, 1, 1);
hist(dati, 20);
title('Istogramma dati')
subplot(2, 1, 2);
normplot(dati);


% prendo i dati 50 alla volta ed eseguo le stesse operazioni
Nblocchi = 8;
lunghezzaBlocco = length(dati)/Nblocchi;    % Nblocchi deve essere scelto in modo che lunghezzaBlocco sia un intero
campioni.dati = zeros(lunghezzaBlocco, Nblocchi);
for i=0:Nblocchi-1
    
    campioni.dati(:,i+1) = dati(i*lunghezzaBlocco+1:(i+1)*lunghezzaBlocco);
    campioni.media(i+1) = mean(campioni.dati(:,i+1));
    campioni.varianza(i+1) = var(campioni.dati(:,i+1));
    campioni.devStandard(i+1) = std(campioni.dati(:,i+1));
    
end

% calcolo la media delle medie
mediaMedie = mean(campioni.media)
% calcolo la media delle varianze
mediaVarianze = mean(campioni.varianza)
% calcolo la media delle deviazione standard
mediaDevStandard = mean(campioni.devStandard)


% verifica della legge secondo la quale, la deviazione standard delle
% medie e' uguale alla deviazione standard dell'intera popolazione
% divisa per la radice quadrata del numero dei campioni
differenza = std(campioni.media) - devStandard / sqrt(400)