% Questo script determina il ritardo tra due segnali, utilizzando la
% correlazione. Inizialmente viene analizzato il caso ideale, e
% successivamente uno piu' realistico.
%
% Per essere eseguito richiede la presenza del Signal Processing Toolbox
%
%
% Autore: Riccardo Riva
% Versione: 1.0
% Data: 18 ottobre 2011


close all; clear all; clc;

% definisco l'intervallo temporale
dt = 1/2000          % tempo fra i campioni [s]
T0 = 0.1             % periodo [s]
t(:,1) = 0:dt:T0;    % istanti di tempo [s]

% creo due impulsi di uguale ampiezza e durata, ma sfalsati
funzione1 = inline('5.*(t>=0.02).*(t<=0.03)', 't');
funzione2 = inline('5.*(t>=0.06).*(t<=0.07)', 't');
segnale1 = funzione1(t);
segnale2 = funzione2(t);

% eseguo la correlazione e determino il ritardo
correlazione = xcorr(segnale1, segnale2, 'coeff');
[dummy, i] = max(correlazione);
ritardo = t(length(t)-i)

% plotto i segnali e la correlazione
figure();
subplot(2, 1, 1);
hold on;
title('Segnali puliti');
plot(t, segnale1, t, segnale2);
legend('Segnale iniziale', 'Segnale finale', ...
       'Location', 'East');
xlabel('t [s]');
ylabel('ampiezza [V]');
subplot(2, 1, 2);
hold on;
title('Correlazione');
plot(correlazione, 'r');
plot(i, correlazione(i), 'ok', 'MarkerEdgeColor', 'k', ...
                               'MarkerFaceColor', 'k', ...
                               'MarkerSize', 5);
xlabel('indice');


% aggiungo del rumore al primo segnale
segnale1N = segnale1 + rand(size(segnale1));
% aggiungo del rumore anche al secondo, ed in piu' ne aumento la durata
funzione3 = inline('5.*(t>=0.06).*(t<=0.08)', 't');
segnale3 = funzione3(t);
segnale2N = segnale3 + rand(size(segnale2));

% eseguo la correlazione
correlazioneN = xcorr(segnale1N, segnale2N, 'coeff');

% plotto i segnali e la correlazione
figure();
subplot(2, 1, 1);
hold on;
title('Segnali con rumore');
plot(t, segnale1N, t, segnale2N);
legend('Segnale iniziale', 'Segnale finale', ...
       'Location', 'East');
xlabel('t [s]');
ylabel('ampiezza [V]');
subplot(2, 1, 2);
hold on;
title('Correlazione');
plot(correlazioneN, 'or', 'MarkerEdgeColor', 'r', ...
                          'MarkerFaceColor', 'r', ...
                          'MarkerSize', 2);
xlabel('indice');

% calcolo il ritardo
maxC = max(correlazioneN);
indici = find(correlazioneN > 0.9*maxC);
tempi = t(length(t)-indici);
ritardoN = mean(tempi)

plot(indici, correlazioneN(indici), 'ok', ...
                                    'MarkerEdgeColor', 'k', ...
                                    'MarkerFaceColor', 'k', ...
                                    'MarkerSize', 3);
